@@ -57,10 +57,11 @@ void Transformation::forceNewArea(double xpcmin, double ypmin, double xpcmax, do
 {
 	
 }
+#define PRINT(w, a) MagLog::dev() << w << "---> " << #a << ": " << (a) << std::endl
 void Transformation::fill(double& width, double& height)
 {
+    init();
 
-	init();
 		// with and height will only
 		double w = getAbsoluteMaxPCX() - getAbsoluteMinPCX();
 		double h = getAbsoluteMaxPCY() - getAbsoluteMinPCY();
@@ -91,175 +92,73 @@ void Transformation::fill(double& width, double& height)
 
 			  	askedWidth_ = width*40;
 			  		askedHeight_ = height*40;
-		
+
+			  		 PRINT ("fill", minx);
+			  		    PRINT ("fill", miny);
+			  		    PRINT ("fill", maxx);
+			  		    PRINT ("fill", maxy);
 			setNewPCBox(minx, miny, maxx, maxy);
 }
 
-#define PRINT(w, a) MagLog::dev() << w << "---> " << #a << ": " << (a) << std::endl
 
 
 void Transformation::tile(double& width, double& height)
 {
-	init();
+	fill(width, height);
+
 	// here we assume we are in the wrepMode, this should be checked!
-
-
-	// with and height will only
-	double awidth = getAbsoluteMaxPCX() - getAbsoluteMinPCX();
-	double aheight = getAbsoluteMaxPCY() - getAbsoluteMinPCY();
+	init();
 
 	double minx =  getAbsoluteMinPCX();
 	double maxx =  getAbsoluteMaxPCX();
 	double miny =  getAbsoluteMinPCY();
 	double maxy =  getAbsoluteMaxPCY();
-
-	awidth = maxx -minx;
-	aheight = maxy -miny;
-
-
-
-
-			int tiles;
-			double nw = (width/height) * aheight;
-
-
-
-
-
-
-
-
-
-
+	  PRINT ("B", minx);
+	    PRINT ("Btile", miny);
+	    PRINT ("Btile", maxx);
+	    PRINT ("Btile", maxy);
+    double pcwidth = maxx - minx;
+    double pcheight = maxy - miny;
 
 	double pxheight = height*40.;
 	double pxwidth = width*40.;
 
-	// Try to see if we have already a number of tile.
-	if (same((pxheight/tile_ - int(pxheight)/tile_), 0)) {
-		pxheight = int(pxheight);
+	tile_ = 512;
+	double tile = 512;
 
-	}
-	if (same((pxwidth/tile_ - int(pxwidth)/tile_), 0)) {
-		pxwidth = int(pxwidth);
+	int xtile = ceil(pxwidth/tile);
+	int ytile = ceil(pxheight/tile);
 
-	}
 
 	askedWidth_ = pxwidth;
 	askedHeight_ = pxheight;
 
-	double pcwidth = maxx-minx;
-	double pcheight = maxy-miny;
-	PRINT("pcwidth", pcwidth);
-	PRINT("pcheight", pcheight);
-	vector<double> levels;
-	double u = unit_;
-	for ( float i = 6; i > 0; i--) {
-
-		levels.push_back((u)/tile_);
-
-
-		u = u/2.;
-	}
-	std::sort(levels.begin(), levels.end());
-
-
-	for ( vector<double>::iterator l = levels.begin(); l != levels.end(); ++l)
-		MagLog::debug()  << *l << ", ";
-
-	double ratio = ( nw > awidth) ? pcheight/pxheight : pcwidth/pxwidth;
-					  	// we need to extend in the x direction
-
-
-
-
-
-
-	vector<double>::iterator l = levels.begin();
-	while ( l != levels.end() ) {
-
-		if ( ratio - *l   <  unitEpsilon_  ) {
-			break;
-		}
-		++l;
-	}
-
-
-
-
-	double level = ( l == levels.end() ) ? levels.back() : *l;
-
-
-	int ytiles = ceil((pcheight/level)/tile_);
-	int xtiles = ceil((pcwidth/level)/tile_);
-
-	zoomLevel_ = ytiles;
-
-
-	pxwidth  = tile_*xtiles;
-	pxheight = tile_*ytiles;
-
-
-	// compute new pxminx, pxminy, pxmaxx, pxmaxy
-
-	int pxminx, pxminy, pxmaxx, pxmaxy;
-
-	pxminx = (minx-originX_)/level;
-	pxminy = (miny-originY_)/level;;
-	pxmaxx = (maxx-originX_)/level;
-	pxmaxy = (maxy-originY_)/level;
-
-	xTile_ = pxminx;
-	yTile_ = pxmaxy;
-
-
-
-	// compute  new minx
-
-	pxminx = floor((float)pxminx/tile_)*tile_;
-	pxminy = floor((float)pxminy/tile_)*tile_;
-
-	pxmaxx =  ceil((float)pxmaxx/tile_)*tile_;
-	pxmaxy =  ceil((float)pxmaxy/tile_)*tile_;
-
-
-
-
-	// now we can compute the new minx,,,
-	minx = (pxminx * level) + originX_;
-	miny =  (pxminy * level) + originY_;
-	maxx = (pxmaxx * level) + originX_;
-	maxy = (pxmaxy * level) + originY_;
-	// now we jsut have to count the number of tiles to find the width and the hieght
-
-	width = (pxmaxx-pxminx);
-	height = (pxmaxy-pxminy);
-
-	if ( askedWidth_ > width ) {
-		maxx += level*tile_;
-		width += tile_;
-
-	}
-	if ( askedHeight_ > height ) {
-
-			miny -= level*tile_;
-			height +=tile_;
-
-	}
-	width /=40.;
-	height /=40.;
-
-
-
-
-	xTile_ = xTile_ - pxminx;
-	yTile_ = pxmaxy - yTile_;
-
+	width = xtile * tile;
+	height = ytile * tile;
 
 
+    double xoffset = ((width - pxwidth)/pxwidth)*pcwidth; 
+    double yoffset = ((height - pxheight)/pxheight)*pcheight; 
+    xoffset /=2;
+    yoffset /=2;
+    
+    minx -= xoffset;
+    miny -= yoffset;
+    maxx += xoffset;
+    maxy += yoffset;
+    xTile_ = (width - pxwidth)/2;
+    yTile_ = (height - pxheight)/2;
+    // Now we fill
 
+    PRINT ("tile", minx);
+    PRINT ("tile", miny);
+    PRINT ("tile", maxx);
+    PRINT ("tile", maxy);
+	width = width/40.;
+	height = height/40.;
 	setNewPCBox(minx, miny, maxx, maxy);
-	init();
+
+
 }
 
 void Transformation::aspectRatio(double& width, double& height)
@@ -346,28 +245,30 @@ bool ViewFilter::in(const PaperPoint& xy){
 
 void Transformation::init()
 {
-	PCEnveloppe_->clear();
-
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMaxPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMaxPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMinPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
+    cleanPCEnveloppe();
 	askedxmin_ =  std::min(getMinPCX(), getMaxPCX());
 	askedxmax_ =  std::max(getMinPCX(), getMaxPCX());
 	askedymin_ =  std::min(getMinPCY(), getMaxPCY());
 	askedymax_ =  std::max(getMinPCY(), getMaxPCY());
 }
 
+void Transformation::cleanPCEnveloppe()
+{
+    PCEnveloppe_->clear();
+        
+        PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
+        PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMaxPCY()));
+        PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMaxPCY()));
+        PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMinPCY()));
+        PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
+    
+    
+}
+
 void Transformation::cleaninit()
 {
-	PCEnveloppe_->clear();
-
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMaxPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMaxPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMinPCY()));
-	PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
+    cleanPCEnveloppe();
+	
 	askedxmin_ =  std::min(getMinPCX(), getMaxPCX());
 	askedxmax_ =  std::max(getMinPCX(), getMaxPCX());
 	askedymin_ =  std::min(getMinPCY(), getMaxPCY());
@@ -413,8 +314,8 @@ void Transformation::visit(MetaDataVisitor& visitor,
 
 	java << "\"pcxmin\" : \"" << getMinPCX() <<  "\",";		
 	java << "\"pcymin\" : \"" << getMinPCY() <<  "\",";		
-	java << "\"pcxmin\" : \"" << getMinPCX() <<  "\",";
-	java << "\"pcymin\" : \"" << getMinPCY() <<  "\",";
+	java << "\"pcxmax\" : \"" << getMaxPCX() <<  "\",";
+	java << "\"pcymax\" : \"" << getMaxPCY() <<  "\",";
 
 	java << "\"pcwidth\" : \"" << w <<  "\",";	
 	java << "\"pcheight\" : \"" << h <<  "\",";
@@ -448,67 +349,109 @@ void Transformation::operator()(const Polyline& from, BasicGraphicsObjectContain
 	PaperPoint ur(getMaxPCX(), getMaxPCY());
 	boost::geometry::model::box<PaperPoint> box(ll, ur);
 	boost::geometry::correct(box);
+
+
+
+
+	vector<deque<PaperPoint> > holes;
+	vector<Polyline*> forholes;
+
 	if ( from.closed() ) {
+	try {
 		deque<PaperPoint> line;
-
+			//line.reserve( from.size());
 		for (unsigned i = 0; i < from.size(); i++) {
-			line.push_back(from.get(i));
+				line.push_back(from.get(i));
 
 		}
-
-		boost::geometry::correct(line);
 		vector<deque<PaperPoint> > result;
-		try {
-			boost::geometry::intersection(box, line, result);
-
-			// Now we feed the graphic container!
-
-			for (vector<deque<PaperPoint> >::iterator l = result.begin(); l != result.end(); l++)
-			{
-				Polyline* poly = from.getNew();
-
-				for (deque<PaperPoint>::iterator point = l->begin(); point != l->end(); ++point)
-					poly->push_back(*point);
-
-				if ( !poly->empty() )
-					out.push_back(poly);
-			}
-		}
-		catch (...) {
-			MagLog::warning() << "error clipping->line ignored" << endl;
-		}
-	}
-	else {
-		vector<PaperPoint> line;
-
-		for (unsigned i = 0; i < from.size(); i++) {
-			line.push_back(from.get(i));
-		}
 		boost::geometry::correct(line);
-		vector<vector<PaperPoint> > result;
-		try {
 		boost::geometry::intersection(box, line, result);
 
 		// Now we feed the graphic container!
 
-		for (vector<vector<PaperPoint> >::iterator l = result.begin(); l != result.end(); l++)
+		for (vector<deque<PaperPoint> >::iterator l = result.begin(); l != result.end(); l++)
 		{
 			Polyline* poly = from.getNew();
 
-			for (vector<PaperPoint>::iterator point = l->begin(); point != l->end(); ++point)
+			for (deque<PaperPoint>::iterator point = l->begin(); point != l->end(); ++point)
 				poly->push_back(*point);
 
-			if ( !poly->empty() )
+			if ( !poly->empty() ) {
 				out.push_back(poly);
-		}
+				forholes.push_back(poly);
+			}
+			// Now we add the holes
 		}
 
-				catch (...) {
-					MagLog::warning() << "error clipping->line ignored" << endl;
+		for (Polyline::Holes::const_iterator h = from.beginHoles(); h !=  from.endHoles(); ++h) {
+
+			deque<PaperPoint> hole;
+			for (deque<PaperPoint>::const_iterator pt = h->begin(); pt != h->end(); ++pt) {
+				hole.push_back(*pt);
+			}
+
+			vector<deque<PaperPoint> > clip;
+			boost::geometry::correct(hole);
+			boost::geometry::intersection(box, hole, clip);
+			for (vector<deque<PaperPoint> >::iterator l = clip.begin(); l != clip.end(); l++) {
+				if ( l->size() ) {
+					holes.push_back(deque<PaperPoint>());
+					for (deque<PaperPoint>::iterator point = l->begin(); point != l->end(); ++point)
+						holes.back().push_back(*point);
 				}
 
+			}
+		}
 
+		for ( vector<Polyline*>::const_iterator poly = forholes.begin(); poly != forholes.end(); ++poly)
+			for (vector<deque<PaperPoint> >::iterator x = holes.begin(); x != holes.end(); x++) {
+
+				if ( (*poly)->within(x->front()) ) {
+					(*poly)->newHole();
+					for (deque<PaperPoint>::iterator point = x->begin(); point != x->end(); ++point)
+						(*poly)->push_back_hole(*point);
+				}
+			}
 	}
+	catch (...) {
+		MagLog::warning() << "error clipping->line ignored" << endl;
+	}
+	return;
+	}
+	try {
+			vector<PaperPoint> line;
+				//line.reserve( from.size());
+			for (unsigned i = 0; i < from.size(); i++) {
+					line.push_back(from.get(i));
+
+			}
+			vector<vector<PaperPoint> > result;
+			boost::geometry::correct(line);
+			boost::geometry::intersection(box, line, result);
+
+			// Now we feed the graphic container!
+
+			for (vector<vector<PaperPoint> >::iterator l = result.begin(); l != result.end(); l++)
+			{
+				Polyline* poly = from.getNew();
+
+				for (vector<PaperPoint>::iterator point = l->begin(); point != l->end(); ++point)
+					poly->push_back(*point);
+				if ( !poly->empty() )
+					out.push_back(poly);
+
+			}
+	}
+
+	catch (...) {
+		MagLog::warning() << "error clipping->line ignored" << endl;
+	}
+
+
+
+
+
 }
 
 
@@ -553,6 +496,7 @@ bool Transformation::in(const UserPoint& point) const
 bool Transformation::in(const PaperPoint& point) const
 {
 	if ( PCEnveloppe_->empty()) {
+        
 		PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMinPCY()));
 		PCEnveloppe_->push_back(PaperPoint(getMinPCX(), getMaxPCY()));
 		PCEnveloppe_->push_back(PaperPoint(getMaxPCX(), getMaxPCY()));
@@ -713,119 +657,113 @@ void useRef(double ref, double inc, double& min, double& max)
 
 }
 
-/* refx, rey,stepx stepy Should in projection coordinates! */
 
-void Transformation::thin(double xstep, double ystep, double gap,  vector<UserPoint>& in) const
+
+
+
+
+
+void Transformation::thin(double step, PaperPoint& origin, vector<pair<double, double> >& points) const
 {
-	Timer timer("thiining", "");
+	Timer timer("thinning", "");
+
+	vector< std::pair<double, double> > xypoints;
+	double minx = getMinPCX();
+	double maxx = getMaxPCX();
+	double miny = getMinPCY();
+	double maxy = getMaxPCY();
+
+
+	std::set<double> xl, yl;
 
 
 
-
-	double uxmin, uxmax, uymin, uymax;
-	int dimx, dimy;
-	// Find the box..
-	uxmin = uxmax = in.front().x_;
-	uymin = uymax = in.front().y_;
-
-
-
-	for ( PointIter pt = in.begin(); pt != in.end(); ++pt ) {
-		double x = pt->x_;
-		double y = pt->y_;
-		if ( x < uxmin ) uxmin = x;
-		if ( y < uymin ) uymin = y;
-		if ( x > uxmax ) uxmax = x;
-		if ( y > uymax ) uymax = y;
+	for ( double x = origin.x(), i = 1; x <= maxx; x = origin.x() + i*step, i++) {
+		if ( x > minx )
+			xl.insert(x);
 	}
+	for ( double x = origin.x(), i = 1; x >= minx; x = origin.x() - i*step, i++)
+		if ( x < maxx )
+			xl.insert(x);
+	for ( double y = origin.y(), i = 1; y <= maxy; y = origin.y() + i*step, i++)
+		if ( y > miny )
+			yl.insert(y);
+	for ( double y = origin.y(), i = 1; y >= miny; y = origin.y() - i*step, i++)
+		if ( y < maxy ) {
+			yl.insert(y);
 
-
-
-
-	dimx = ((uxmax - uxmin)  / xstep ) +1;
-	dimy = ((uymax - uymin ) / ystep ) +1;
-	vector<bool> mask(dimx*dimy, false);
-	for ( int j = 0; j < dimy; j++)	{
-		double lasty = uymin +(ystep*j);
-		double lastx = uxmin;
-		fast_reproject(lastx, lasty);
-		for ( int i = 0; i < dimx; i++) {
-			double y = uymin +(ystep*j);
-			double x = uxmin +(xstep*i);
-			fast_reproject(x, y);
-			if ( ( (x-lastx)*(x-lastx) + (y-lasty)*(y-lasty)) < gap )
-				mask[j*dimx+i] = true;
-			else {
-				lastx = x;
-				lasty = y;
-			}
 		}
-	}
+	for (std::set<double>::iterator x = xl.begin(); x != xl.end(); ++x)
+		for (std::set<double>::iterator y = yl.begin(); y != yl.end(); ++y){
+							xypoints.push_back(make_pair(*x, *y));
+					}
 
 
-	vector<PointIter> helper(dimx*dimy, in.end());
+	revert(xypoints, points);
 
-	for ( PointIter pt = in.begin(); pt != in.end(); ++pt ) {
-		double x = pt->x_;
-		double y = pt->y_;
+}
 
+void Transformation::thin(double step, PaperPoint& origin, Matrix& matrix, double missing) const
+{
+/*
+	Timer timer("thinning", "");
 
+	vector< std::pair<double, double> > xypoints;
+	double minx = getMinPCX();
+	double maxx = getMaxPCX();
+	double miny = getMinPCY();
+	double maxy = getMaxPCY();
 
-		int xx = ceil(x-uxmin)/xstep;
-		int yy = ceil(y-uymin)/ystep;
 
+	std::set<double> xl, yl;
 
 
-		vector<pair<int, int> > check;
 
-		check.push_back(make_pair(yy, xx));
-		//if ( xx + 1 < dimx )
-			//check.push_back(make_pair(yy, xx+1));
-		//if ( yy + 1 < dimy )
-			//check.push_back(make_pair(yy+1, xx));
-		//if ( xx + 1 < dimx && yy + 1 < dimy )
-			//check.push_back(make_pair(yy+1, xx+1));
+	for ( double x = origin.x(), i = 1; x <= maxx; x = origin.x() + i*step, i++) {
+		if ( x > minx )
+			xl.insert(x);
+	}
+	for ( double x = origin.x(), i = 1; x >= minx; x = origin.x() - i*step, i++)
+		if ( x < maxx )
+			xl.insert(x);
+	for ( double y = origin.y(), i = 1; y <= maxy; y = origin.y() + i*step, i++)
+		if ( y > miny )
+			yl.insert(y);
+	for ( double y = origin.y(), i = 1; y >= miny; y = origin.y() - i*step, i++)
+		if ( y < maxy ) {
+			yl.insert(y);
 
-
-
-		for ( vector<pair<int, int> >::iterator ind = check.begin(); ind != check.end(); ++ind) {
-			int i = ind->first*dimx + ind->second;
-			if ( mask[i] )
-				continue;
-			if ( i < 0 || i >= dimx*dimy)
-				continue;
-
-			double x0 = ind->second * xstep + uxmin;
-			double y0 = ind->first * ystep + uymin;
-			if ( !this->in(x0, y0) )
-				continue;
+		}
 
+	matrix.set(x.size(), y.size(), missing);
 
-			PointIter last = helper[i];
+	for ( std::set<double>::iterator x = xl.begin(), x != xl.end(); ++x )
+		matrix.columns().push_back(*x);
+	for ( std::set<double>::iterator y = yl.begin(), y != yl.end(); ++y )
+			matrix.columns().push_back(*y);
 
-			if ( last == in.end()) {
-				helper[i ]= pt;
-			}
-			else {
+	(*matrix)->setMapsAxis();
+*/
+}
 
-				double dist = sqrt(((x-x0) * (x-x0)) +  ((y - y0) * (y - y0)));
+UserPoint Transformation::reference() const
+{
+	PaperPoint xy((getMinPCX() +  getMaxPCX())/2., (getMinPCY() +  getMaxPCY())/2.);
+	UserPoint ll;
+	revert(xy, ll);
+	return ll;
+}
 
-				double lastx = last->x_;
-				double lasty = last->y_;
+double Transformation::distance(UserPoint& point1, UserPoint& point2) const
+{
+	double x1 = point1.x_;
+	double y1 = point1.y_;
+	double x2 = point2.x_;
+	double y2 = point2.y_;
+	fast_reproject(x1, y1);
+	fast_reproject(x2, y2);
 
-				double min = sqrt(((lastx-x0) * (lastx-x0)) +  ((lasty - y0) * (lasty -y0)));
-				if (dist < min){
-					helper[i ] = pt;
-				}
-			}
-		}
-
+	return sqrt (( x1-x2 ) *  ( x1-x2 ) + ( y1-y2 ) *  ( y1-y2 ));
 
-	}
-
-	for ( int j = 0; j < dimx*dimy; j++) {
-		if ( helper[j] != in.end() )
-			helper[j]->flagMissing();
-	}
 
 }
