@@ -1016,11 +1016,14 @@ MAGICS_NO_EXPORT void PostScriptDriver::renderText(const Text& text) const
 		}
 
 		const std::set<string>& styles = magfont.styles();
+
 		string style = "";
 		if(styles.find("bold") != styles.end()) style = "bold";
-		if(styles.find("italic") != styles.end()) style += "italic";
+		else if(styles.find("italic") != styles.end()) style += "italic";
+		else if(styles.find("bolditalic") != styles.end()) style += "bolditalic";
 		if(style == "") style = "normal";
 		const string lowFont = lowerCase(magfont.name()+"_"+style);
+
 		fontMapIter iter = FontMap_.find(lowFont);
 
 		const bool underlined = (styles.find("underlined") != styles.end()) ? true : false;
@@ -1031,7 +1034,7 @@ MAGICS_NO_EXPORT void PostScriptDriver::renderText(const Text& text) const
 		else
 		{
 			font = 0; // if not found get default
-			MagLog::info() << "PostScriptDriver: Font "<< lowFont << " is not registered!\n   Default font used for "<<(*niceText).text()<<"."<< endl;
+			MagLog::warning() << "PostScriptDriver: Font "<< lowFont << " is not registered!\n   Default font used for "<<(*niceText).text()<<"."<< endl;
 		}
 
 		setNewColour(magfont.colour());
@@ -1209,7 +1212,7 @@ MAGICS_NO_EXPORT bool PostScriptDriver::renderPixmap(MFloat x0,MFloat y0,MFloat
 		switch ( col_model )
 		{
 			 case 0:
-				 sprintf(t,"%02hx%02hx%02hx",r,g,b);
+				 sprintf(t,"%02hhx%02hhx%02hhx",r,g,b);
 				 break;
 			 case 1:
 				 cc = 1. - (r*0.00392156); cm = 1. - (g*0.00392156); cy = 1. - (b*0.00392156);
@@ -1238,7 +1241,7 @@ MAGICS_NO_EXPORT bool PostScriptDriver::renderPixmap(MFloat x0,MFloat y0,MFloat
 				 sprintf(t,"%02hx%02hx%02hx",kr,kr,kr);
 				 break;
 			 default:
-				 sprintf(t,"%02hx%02hx%02hx",r,g,b);
+				 sprintf(t,"%02hhx%02hhx%02hhx",r,g,b);
 				 break;
 		}
 		*ps << t;
