@@ -10,6 +10,7 @@
 #include <AxisWrapper.h>
 #include <VisualAction.h>
 #include "MetaDataWrapper.h"
+#include "ImportObjectHandlerWrapper.h"
 
 #ifdef MAGICS_GRIB
 #include <GribDecoderWrapper.h>
@@ -157,7 +158,7 @@ MagPlus::MagPlus() : root_(0), superpage_(-1), geographical_(true), mode_(intera
  		sceneCreators_["PAGE"] = &MagPlus::page;
  		sceneCreators_["NEWPAGE"] = &MagPlus::newpage;
  		sceneCreators_["MCOAST"] = &MagPlus::coastlines;
- 		sceneCreators_["MTEPHI"] = &MagPlus::tephigrid;
+ 		sceneCreators_["MTHERMOGRID"] = &MagPlus::tephigrid;
  		sceneCreators_["PCOAST"] = &MagPlus::oldcoastlines;
 
 
@@ -222,7 +223,7 @@ MagPlus::MagPlus() : root_(0), superpage_(-1), geographical_(true), mode_(intera
         sceneCreators_["MTEXT"] = &MagPlus::text;
         sceneCreators_["MLEGEND"] = &MagPlus::legend;
         sceneCreators_["DEVICE"] = &MagPlus::device;
-        sceneCreators_["PIMPORT"] = &MagPlus::import; 
+        sceneCreators_["MIMPORT"] = &MagPlus::import;
         sceneCreators_["PRASTER"] = &MagPlus::raster;
         sceneCreators_["PRASTERLOOP"] = &MagPlus::rasterloop;
         sceneCreators_["BINNING_OBJECT"] = &MagPlus::binning;
@@ -336,7 +337,7 @@ bool MagPlus::qtdriver(magics::MagRequest& /*in*/)
 	{
 		assert(qtScene_);
 		qtDriver_ = new magics::QtDriver();
-        qtDriver_->scene_= qtScene_;
+        qtDriver_->setScene(qtScene_);
 
 	}
 #ifndef MAG_NEXT
@@ -577,7 +578,7 @@ bool MagPlus::cartesianGrid(magics::MagRequest& in) {
 }
 bool MagPlus::tephiGrid(magics::MagRequest& in)
 {
-	magics::MagRequest& tephi = in.getSubRequest("TEPHIGRAM_GRID");
+	magics::MagRequest& tephi = in.getSubRequest("THERMO_GRID");
 	if ( tephi ) {
 
 		// use the user defined one
@@ -592,7 +593,7 @@ bool MagPlus::tephiGrid(magics::MagRequest& in)
 	}
 	else {
 		TephiGrid* grid = new TephiGrid();
-		grid->icon("Tephigram Grid", "MTEPHI");
+		grid->icon("Tephigram Grid", "MTHERMO_GRID");
 		top()->push_back(grid);
 
 	}
@@ -646,8 +647,8 @@ bool MagPlus::tephigrid(magics::MagRequest& in)
 	MagLog::dev()<< "add Tephi Grid" << endl;
 
 
-	replace_string(in, "_NAME", "", "Tephigram");
-	replace_string(in, "_CLASS", "", "MTEPHI");
+	replace_string(in, "_NAME", "", "Thermogrid");
+	replace_string(in, "_CLASS", "", "MTHERMOGRID");
 	TephiGridWrapper helper;
 
 	helper.set(in);
@@ -679,9 +680,12 @@ bool MagPlus::axis(magics::MagRequest& in)
 	return false; // do not exit
 }
 
-bool MagPlus::import(magics::MagRequest& /*in*/)
+bool MagPlus::import(magics::MagRequest& in)
 {
-	MagLog::dev()<< "ignore for now!!!" << endl;
+
+	ImportObjectHandlerWrapper object;
+	object.set(in);
+	top()->push_back(object.object());
 	return false; // do not exit
 }
 
@@ -1200,6 +1204,8 @@ void checknctypes() {
 	        nctypes["NETCDF_XY_POINTS"] = "xypoint";
 	        nctypes["NETCDF_XY_VECTORS"] = "vector";
 	        nctypes["NETCDF_XY_MATRIX"] = "matrix";
+	        nctypes["ARRAY"] = "matrix";
+	        nctypes["MATRIX"] = "complex_matrix";
 
 	        nctypes["NETCDF_GEOPOINTS"] = "geopoint";
 	        nctypes["NETCDF_GEOVECTORS"] = "geovector";
@@ -1224,8 +1230,8 @@ bool MagPlus::netcdf(magics::MagRequest& in)
 	in("NETCDF_FILENAME") = path.c_str();
 	    static map<string, string> types;
 
-	string type = get(in, "NETCDF_PLOT_TYPE", in.getVerb());
-	in("NETCDF_TYPE") = nctypes[type].c_str();
+    string type = get(in, "NETCDF_POSITION_TYPE", in.getVerb());
+   	in("NETCDF_TYPE") = nctypes[type].c_str();
 
 	VisualAnimation* action = new VisualAnimation();
 	top()->push_back(action);
