@@ -6,7 +6,7 @@ from types import *
 import sys
 
 
-if(len(sys.argv) != 5) :
+if(len(sys.argv) != 6) :
     print "\n\tYou need to give 4 input parameters:"
     print "\n\t  %s source.xml targetDef CLASS_NAME rulesDef\n" % sys.argv[0]
     sys.exit()
@@ -53,8 +53,6 @@ class ObjectHandler(ContentHandler):
 	ignore = False
 	mydef = []   # the growing list of parameters
 	param = ""
-	tag = ""
-	doc = "Documentation"
 	myoptions = []
 	optionalparams  = {}
 	optionalparams2 = {}  # see reorganiseOptionalParameters()
@@ -65,7 +63,6 @@ class ObjectHandler(ContentHandler):
 	classes = {}
 	#inheritedlast = ""
 	unset = {}
-	current = {}
 	filehistory = []  # a list of files we have opened (so that we don't open the same one twice)
 	debug = 0  # gives some output on stdout
 	toplevel = True
@@ -87,6 +84,10 @@ class ObjectHandler(ContentHandler):
 			return "-1.0E21"
 		if (val.upper() == "INT_MAX"):
 			return "1.0E21"
+		if (val.upper() == "-LLONG_MAX"):
+			return "-1.0E21"
+		if (val.upper() == "LLONG_MAX"):
+			return "1.0E21"
 		if (val == "floatarray()"):
 			return "''"
 		if (val == "intarray()"):
@@ -115,11 +116,8 @@ class ObjectHandler(ContentHandler):
 			self.file.write(" %s='%s'" % (key.upper() , attrs.getValue(key).upper()))
 		self.file.write(">\n")
 
-	def characters(self, content):
-		
-		if ( self.tag == "documentation" ) : 				
-				self.doc += content.lstrip();
-			
+	def characters(self, data):
+		pass
 	
 
 	# addhidden - if the appropriate flag is set in the attributes, return the text that will
@@ -131,7 +129,7 @@ class ObjectHandler(ContentHandler):
 			return ""
 
 	def toggle(self, attrs):
-		s = "\t%s [ help_text = '%s' ]\n %s\t{\n" % (attrs.get("name").upper(), self.doc.strip(), self.addhidden(attrs))
+		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
 		s = s + "\t\tON;  ON\n"
 		s = s + "\t\tOFF; OFF\n"
 		s = s + "\t} = %s\n" % self.default(attrs).upper()
@@ -165,7 +163,7 @@ class ObjectHandler(ContentHandler):
 
 
 	def number(self, attrs):
-		s = "\t%s[ help_text = '%s' ]\n%s\t{\n" % (attrs.get("name").upper(), self.doc.strip(), self.addhidden(attrs))
+		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
 		s = s + "\t\t*\n"
 		s = s + "\t} = %s\n" % self.default(attrs).upper()
 		return s
@@ -182,12 +180,12 @@ class ObjectHandler(ContentHandler):
 
 	def listofstrings(self, attrs):
 		if (attrs.get("colourlist") == "on" ) :
-			 s = "\t%s [ help = help_colour,interface = colour, help_text='%s' ]\n%s\t{ \n" % (attrs.get("name").upper(), self.doc.strip(), self.addhidden(attrs))
+			 s = "\t%s [ help = help_colour,interface = colour ]\n%s\t{ \n" % (attrs.get("name").upper(), self.addhidden(attrs))
 			 s = s + "\t\t&PARAMSHARE&COLOUR"
 			 s = s + "\t\t/\n"
 			 s = s + "\t} = %s\n" % self.default(attrs).upper()
 		elif (attrs.get("countrylist") == "on" ):
-			s = "\t%s [ help = help_multiple_selection, exclusive = False, help_text='%s']\n%s\t{\n" % (attrs.get("name").upper(), self.doc.strip(), self.addhidden(attrs) )
+			s = "\t%s [ help = help_multiple_selection, exclusive = False ]\n%s\t{\n" % (attrs.get("name").upper(),self.addhidden(attrs) )
 			countries = attrs.get("values").split("/")
 			for c in countries:
 				parts = c.split(':')
@@ -209,7 +207,7 @@ class ObjectHandler(ContentHandler):
 
 
 	def colour(self, attrs):
-		s = "\t%s [ help = help_colour,interface = colour, help_text='%s' ]\n%s\t{ \n" % (attrs.get("name").upper(), self.doc.strip(), self.addhidden(attrs))
+		s = "\t%s [ help = help_colour,interface = colour ]\n%s\t{ \n" % (attrs.get("name").upper(), self.addhidden(attrs))
 		s = s + "\t\t&PARAMSHARE&COLOUR\n"
 		mi = attrs.get("metview_interface")
 		if mi == "ColourWithExtra":
@@ -256,8 +254,7 @@ class ObjectHandler(ContentHandler):
 		return s.upper()
 	def font(self, attrs):
 		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
-		s = s + "\t\tserif;serif\n"
-		s = s + "\t\tsansserif;sansserif\n"
+		s = s + "\t\t&PARAMSHARE&FONT\n"
 		s = s + "\t} = %s\n" % self.default(attrs).upper()
 		return s.upper()
 
@@ -513,7 +510,6 @@ class ObjectHandler(ContentHandler):
 
 
 	def startElement(self, name, attrs):
-		self.tag = name;
 		if (name == "magics") :
 			return
 
@@ -531,7 +527,7 @@ class ObjectHandler(ContentHandler):
 			if doc_inherits != 'no' and self.inherits != '' and self.inherits != None:
 				self.classes[self.classname]["inherits_params_from"].add(self.inherits)
 				self.classes[self.classname]["inherits_reqs_from"].add(self.inherits)
-				fname = "xml/%s.xml" % self.inherits
+				fname = "%s/%s.xml" % (sys.argv[1], self.inherits)
 				if not (fname in self.filehistory):  # don't open the same file twice
 					try:
 						file = open(fname, "r")
@@ -560,18 +556,32 @@ class ObjectHandler(ContentHandler):
 						pass
 
 		if (name == "parameter") or (name == "metview_parameter"):
-		
 			self.ignore = False
 			self.param = attrs.get("name")
-			self.current = attrs
 			if (self.debug) :
 				print("  param: " + self.param)
-		   
-			
-				
-		if (name == "documentation"):
-			self.doc = "";
-		
+			if (attrs.get("implemented") == 'no'):
+				return
+			if (attrs.get("metview") == 'no'):
+				return
+			if (attrs.get("visible") == 'no') or (attrs.get("visible") == 'off') or (attrs.get("visible") == 'false'):
+				return
+			if (attrs.get("inherit_parent_reqs") != 'no'):
+				docclass = attrs.get("doc_class", None)
+				if docclass <> None:
+					paramclass = docclass
+				else:
+					paramclass = self.classname
+				self.addParameterToClass(self.param, paramclass)
+			type = attrs.get("to")
+			metview_type = attrs.get("metview_interface")
+			if metview_type <> None:
+				type = metview_type
+			if (self.types.has_key(type)):
+				f = self.types[type]
+				self.newparam(self.param, f(self, attrs), self.default(attrs))
+			else:
+				self.last = self.last + self.options(attrs)
 
 		if (name == "set"):
 			setname = attrs.get("name")
@@ -596,9 +606,9 @@ class ObjectHandler(ContentHandler):
 					#	print "YClass " + attrs.get("name") + " inherits_reqs_from " + self.classname
 					if attrs.get("docdive") != 'no' and  attrs.get("doc_inherits") != 'no' :
 						if ( not(isinstance(attrs.get("xmlfile"), NoneType))):
-							fname = "xml/%s.xml" % attrs.get("xmlfile")
+							fname = "/%s.xml" % attrs.get("xmlfile")
 						else:
-							fname = "xml/%s.xml" % attrs.get("name")
+							fname = "%s/%s.xml" % (sys.argv[1], attrs.get("name"))
 						if not (fname in self.filehistory):  # don't open the same file twice
 							try:
 								file = open(fname, "r")
@@ -650,37 +660,10 @@ class ObjectHandler(ContentHandler):
 
 		if self.ignore:
 			return
-	
-	
-		if (name == "parameter" or name == "metview_parameter") :
+
+		if ((name == "parameter" or name == "metview_parameter") and len(self.myoptions) != 0) :
 		#if (name == "parameter") :
-			print "param"
-			print(self.current)
-			attrs = self.current;
-			
-			if (attrs.get("implemented") == 'no'):
-				return
-			if (attrs.get("metview") == 'no'):
-				return
-			if (attrs.get("visible") == 'no') or (attrs.get("visible") == 'off') or (attrs.get("visible") == 'false'):
-				return
-			if (attrs.get("inherit_parent_reqs") != 'no'):
-				docclass = attrs.get("doc_class", None)
-				if docclass <> None:
-					paramclass = docclass
-				else:
-					paramclass = self.classname
-				self.addParameterToClass(self.param, paramclass)
-			type = attrs.get("to")
-			metview_type = attrs.get("metview_interface")
-			if metview_type <> None:
-				type = metview_type
-			if (self.types.has_key(type)):
-				f = self.types[type]
-				self.newparam(self.param, f(self, attrs), self.default(attrs))
-			else:
-				self.last = self.last + self.options(attrs)
-				
+			#print(self.myoptions)
 			self.last = self.last + "\t} = %s\n" % self.defparam
 			self.newparam(self.param, self.last, self.defparam)
 			if (self.debug) :
@@ -725,7 +708,7 @@ class ObjectHandler(ContentHandler):
 			if self.toplevel:
 				#print "SL (end class): " + self.last
 				#print('opt: ', self.optionalparams)
-				definition = open(sys.argv[2], "w")
+				definition = open(sys.argv[3], "w")
 				definition.write("PARAMSHARE ; ParamShare; PARAMSHARE\n");
 				definition.write("{\n\tCOLOUR {\n\t\t%include MagicsColors.h\n\t}\n")
 				definition.write("\tSTYLE {\n")
@@ -735,8 +718,18 @@ class ObjectHandler(ContentHandler):
 				definition.write("\t\tCHAIN_DOT; CHAIN_DOT\n")
 				definition.write("\t\tCHAIN_DASH; CHAIN_DASH\n")
 				definition.write("\t}\n")
+				definition.write("\tFONT {\n")
+				definition.write("\t\tARIAL; ARIAL\n")
+				definition.write("\t\tCOURIER; COURIER\n")
+				definition.write("\t\tHELVETICA; HELVETICA\n")
+				definition.write("\t\tTIMES; TIMES\n")
+				definition.write("\t\tSERIF; SERIF\n")
+				definition.write("\t\tSANSSERIF; SANSSERIF\n")
+				definition.write("\t\tSYMBOL; SYMBOL\n")
+				definition.write("\t}\n")
 				definition.write("}\n\n")
-				definition.write("%s; Magics; Automatically generated\n{\n" % sys.argv[3])
+
+				definition.write("%s; Magics; Automatically generated\n{\n" % sys.argv[4])
 				for param in self.mydef:
 					#print('PARAM START')
 					#print(param[1])
@@ -774,7 +767,7 @@ class ObjectHandler(ContentHandler):
 
 				# two different ways to generate the rules
 				if False:
-					rules = open(sys.argv[4], "w")
+					rules = open(sys.argv[5], "w")
 					for optparam in self.optionalparams.keys():
 						rules.write("\n%if")
 						first = True
@@ -803,7 +796,7 @@ class ObjectHandler(ContentHandler):
 					#print "\n\n..............................\n"
 					#print self.optionalparams3
 
-					rules = open(sys.argv[4], "w")
+					rules = open(sys.argv[5], "w")
 					#for conditions in self.optionalparams2:
 					for clause in self.optionalparams3:
 						#print "CLAUSE"
@@ -850,6 +843,6 @@ object = ObjectHandler()
 saxparser = make_parser()
 saxparser.setContentHandler(object)
 
-datasource = open(sys.argv[1], "r")
+datasource = open(sys.argv[1] +"/" + sys.argv[2], "r")
 saxparser.parse(datasource)
 print "DONE"
